/* Header for use in defining a given protocol for state tracking. */
#ifndef _IP_STATE_PROTOCOL_H
#define _IP_STATE_PROTOCOL_H

#include "ip_state.h"

struct iphdr;

struct ip_state_protocol
{
	/* Next pointer. */
	struct ip_state_protocol *next;

	/* Protocol number. */
	u_int8_t proto;

	/* Protocol name */
	const char *name;

	/* Minimum packet length after IP header. */
	size_t min_length;

	/* Try to fill in the third arg; return true if possible. */
	int (*pkt_to_tuple)(const void *datah, size_t datalen,
			    struct ip_state_tuple *tuple);

	/* Invert the per-proto part of the tuple: ie. turn xmit into reply. 
	 * Some packets can't be inverted: return 0 in that case.
	 */
	int (*invert_tuple)(struct ip_state_tuple *inverse,
			    const struct ip_state_tuple *orig);

	/* Print out the per-protocol part of the tuple. */
	unsigned int (*print_tuple)(char *buffer,
				    const struct ip_state_tuple *);

	/* Print out the private part of the state. */
	unsigned int (*print_state)(char *buffer, const struct ip_state *);

	/* Called before state is destroyed. */
	void (*destroy)(struct ip_state *state);

	/* Returns verdict for packet, and may modify statetype */
	unsigned int (*established)(struct ip_state *state,
				    struct iphdr *iph, size_t len,
				    int direction, 
				    enum ip_statetype *statetype);

	/* Called when a new connection for this protocol found. */
	unsigned int (*new)(struct ip_state *state,
			    struct iphdr *iph, size_t len,
			    enum ip_statetype *statetype);

	/* Time (in HZ) until expiry.  0 = no expiry. */
	unsigned long expiry;
};

/* Find a given protocol */
extern struct ip_state_protocol *find_proto(u_int8_t protocol);

/* Protocol registration. */
extern int ip_state_register_protocol(struct ip_state_protocol *proto);
extern void ip_state_unregister_protocol(struct ip_state_protocol *proto);

extern void ip_state_protocols_init();
#endif /*_IP_STATE_PROTOCOL_H*/
