#ifndef _IP_NAT_H
#define _IP_NAT_H
#include "../state/ip_state.h"

#define IP_NAT_MAPPING_TYPE_MAX_NAMELEN 16

enum ip_nat_manip_type
{
	IP_NAT_MANIP_SRC,
	IP_NAT_MANIP_DST
};

/* Hashes for by-source and IP/protocol. */
struct ip_nat_hash
{
	struct ip_nat_hash *next;

	/* state we're embedded in: NULL if not in hash. */
	struct ip_state *state;
};

/* The manipulations for a specific connection. */
struct ip_nat_manipulation
{
	/* The type of manipulation to perform */
	enum ip_nat_manip_type maniptype;

	/* Replacement. */
	struct ip_state_manip manip;
};

/* At most two manipulations. */
#define IP_NAT_MAX_MANIPS 2

/* The structure embedded in the state structure. */
struct ip_nat_info
{
	/* Set to zero when state created. */
	int initialized;

	/* Indexed by state direction. */
	struct {
		/* Which hook the manipulation is to take place. */
		unsigned int hook;
		/* Number of manipulations. */
		unsigned int num_manips;
		/* Manipulations to occur at each state in this dirn. */
		struct ip_nat_manipulation manipulation[IP_NAT_MAX_MANIPS];
	} dirinfo[2];

	/* The mapping type which created us (NULL for null mapping). */
	const struct ip_nat_mapping_type *mtype;

	struct ip_nat_hash bysource, byipsproto;
};

#define IP_NAT_RANGE_MAP_IPS 1
#define IP_NAT_RANGE_PROTO_SPECIFIED 2
/* Used internally by get_unique_tuple(). */
#define IP_NAT_RANGE_FULL 4

/* Single range specification. */
struct ip_nat_range
{
	/* Set to OR of flags above. */
	unsigned int flags;

	/* Inclusive: network order. */
	u_int32_t min_ip, max_ip;

	union ip_state_manip_proto min, max;
};

/* A range consists of an array of 1 or more ip_nat_range */
struct ip_nat_multi_range
{
	unsigned int rangesize;

	/* hangs off end. */
	struct ip_nat_range range[1];
};

/* Is this tuple already taken? (not by us)*/
extern int ip_nat_used_tuple(const struct ip_state_tuple *tuple,
			     const struct ip_state *ignored_state);

/* Calculate relative checksum. */
extern u_int16_t ip_nat_cheat_check(u_int32_t oldvalinv,
				    u_int32_t newval,
				    u_int16_t oldcheck);
#endif
