#ifndef _IP_NAT_CORE_H
#define _IP_NAT_CORE_H
/* This header used to share core functionality between the standalone
   NAT module, and the compatibility layer's use of NAT for masquerading. */
extern void ip_nat_init(void);
extern void ip_nat_cleanup(void);

extern size_t ip_nat_offset;

#define IP_NAT_HTABLE_SIZE 64

extern struct ip_nat_hash *bysource[IP_NAT_HTABLE_SIZE];
extern struct ip_nat_hash *byipsproto[IP_NAT_HTABLE_SIZE];

/* We keep extra hashes for each state, for fast searching. */
extern inline size_t 
hash_by_ipsproto(u_int32_t src, u_int32_t dst, u_int16_t proto)
{
	/* Modified src and dst, to ensure we don't create two
           identical streams. */
	return (src + dst + proto) % IP_NAT_HTABLE_SIZE;
}

extern inline size_t 
hash_by_src(const struct ip_state_manip *manip, u_int16_t proto)
{
	/* Original src, to ensure we map it consistently if poss. */
	return (manip->ip + manip->u.all + proto) % IP_NAT_HTABLE_SIZE;
}

extern void do_binding(struct ip_nat_info *info, int dir, 
		       struct iphdr *iph, size_t len);

extern struct ip_nat_protocol *protos;

extern void icmp_reply_translation(struct sk_buff *skb, 
				   struct ip_state *state,
				   int conn_dir);

extern unsigned int ip_nat_setup_info(struct ip_state *state,
				      enum ip_nat_manip_type maniptype,
				      struct ip_nat_multi_range *mr,
				      const struct device *device);

/* Built-in protocols. */
extern struct ip_nat_protocol ip_nat_protocol_tcp;
extern struct ip_nat_protocol ip_nat_protocol_udp;
extern struct ip_nat_protocol ip_nat_protocol_icmp;
#endif /* _IP_STATE_CORE_H */
