/* Kernel module to match ICMP parameters. */
#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/icmp.h>

#include "ipt_icmp.h"
#include "ip_tables.h"

/* Returns 1 if the type and code is matched by the range, 0 otherwise */
static inline int
type_code_match(u_int8_t test_type, u_int8_t min_code, u_int8_t max_code,
		u_int8_t type, u_int8_t code,
		int invert)
{
	return (type == test_type && code >= min_code && code <= max_code)
		^ invert;
}

static int
match(struct sk_buff *skb,
      const char *indev,
      const char *outdev,
      const union ipt_matchinfo *matchinfo,
      int offset,
      const void *hdr,
      u_int16_t datalen)
{
	const struct icmphdr *icmp = hdr;

	/* Must be long enough to examine, and not a fragment. */
	return datalen >= 2
		&& !offset
		&& type_code_match(matchinfo->icmp.type,
				   matchinfo->icmp.code[0],
				   matchinfo->icmp.code[1],
				   icmp->type, icmp->code,
				   !!(matchinfo->icmp.invflags&IPT_ICMP_INV));
}

/* Called when user tries to insert an entry of this type. */
static int
checkentry(const struct ipt_ip *ip,
	   const union ipt_matchinfo *matchinfo)
{
	/* Must specify proto == ICMP, and no unknown invflags */
	return ip->proto == IPPROTO_ICMP
		&& !(matchinfo->icmp.invflags & ~IPT_ICMP_INV);
}

static struct ipt_match icmp_match
= { NULL, "icmp", &match, &checkentry, &__this_module };

int init_module(void)
{
	return ipt_register_match(&icmp_match);
}

void cleanup_module(void)
{
	ipt_unregister_match(&icmp_match);
}
