#ifndef _IP_NAT_BIND_H
#define _IP_NAT_BIND_H
#include "ip_nat.h"

#define IP_NAT_BASE_CTL  	80	/* base for NAT socket options */
#define IP_NAT_SO_INSERT	IP_NAT_BASE_CTL
#define IP_NAT_SO_DELETE	(IP_NAT_BASE_CTL+1)
#define IP_NAT_SO_MAX		IP_NAT_SO_DELETE

#define IP_NAT_BIND_MAX_NAMELEN 16

struct ip_nat_bind_user
{
	struct ip_nat_tuple_proto tupleproto;

	unsigned int hook;

	/* Set ifname to "" for INCOMING from local process. */
	char ifname[IFNAMSIZ];

	/* Marks which bits we care about. */
	struct ip_nat_pktinfo mask;

	/* Range to map into */
	struct ip_nat_range range;

	/* Hangs off end.  Double NUL char to terminate. */
	char typenames[0];
};

#ifdef __KERNEL__
struct iphdr;

struct ip_nat_perconn;

struct ip_nat_binding_element
{
	/* Here's what we match. */
	struct ip_nat_pktinfo info;
	struct ip_nat_pktinfo mask;

	/* Our pre-calculated precedence number. */
	u_int32_t precedence;

	/* We keep this because pointer in ip_nat_pktinfo gets NULLed on
	 * device down. */
	char ifname[IFNAMSIZ];

	/* NULL-terminated array of bind types hang off the end. */
	struct ip_nat_bind_type *type[0];
};

struct ip_nat_binding
{
	struct ip_nat_binding *next;

	/* Range to map onto (linked list) */
	struct ip_nat_multi_range *mr;

	struct ip_nat_binding_element elem;
};

struct ip_nat_bind_type
{
	struct ip_nat_bind_type *next;

	/* Name of this mapping. */
	const char name[IP_NAT_BIND_MAX_NAMELEN];

	/* Must be zero to unregister. */
	unsigned int usage;

	/* If you want your module count modified as rules use the type,
	   set this. */
	struct module *me;

	/* The size of the private area you want in both ip_nat_perconn. */
	size_t private_size;

	/* Filled in on registration: your offset from start of
	   ip_nat_perconn. */
	size_t private_offset;

	/* Function to print out private part of perconn (if any). */
	unsigned int (*print_private)(char *buffer,
				      const struct ip_nat_perconn *perconn);

	/* Will it accept this bind? */
	int (*check)(const struct ip_nat_binding *, unsigned int typenum);

	/* Function to call if we get a match: if NF_ACCEPT and
	 * perconn filled in, that perconn will be called.  Call next
	 * typenumber's bind.  */
	unsigned int (*bind)(const struct iphdr *iph,
			     const struct ip_nat_pktinfo *pkt,
			     const struct ip_nat_binding_element *cause,
			     struct ip_nat_multi_range *mr,
			     unsigned int typenumber,
			     struct ip_nat_perconn **perconn);

	/* Function called when a perconn is about to be destroyed. */
	void (*destroy)(const struct ip_nat_perconn *perconn);
};

extern int ip_nat_register_bind_type(struct ip_nat_bind_type *me);
extern int ip_nat_unregister_bind_type(struct ip_nat_bind_type *me);

#endif /*__KERNEL__*/
#endif /*_IP_NAT_BIND_H*/
