/* Helper functions. */
#include <stdio.h>
#include <dlfcn.h>
#include <getopt.h>
#include <stdlib.h>

struct ip_nat_bind_user;

struct option *
try_load_library(const char *basename,
		 const char *lib, 
		 void (**help)(void), 
		 void (**parse)(int, char **, struct ip_nat_bind_user *),
		 struct option *original_opts)
{
	/* For the given protocol see if there's a library for it. */
	void *dlp;
	unsigned int num_new_opts, num_orig_opts;
	char path[strlen(basename) + strlen(lib) + strlen(".so") + 1];
	struct option *new_options;
	struct option *opt;

	sprintf(path, "%s%s.so", basename, lib);

	dlp = dlopen(path, RTLD_NOW);
	if (!dlp) {
		fprintf(stderr, "Failed `%s': %s\n", path, dlerror());
		return original_opts;
	}

	*help = dlsym(dlp, "help");
	*parse = dlsym(dlp, "parse");
	new_options = dlsym(dlp, "options");

	for (num_orig_opts = 0; 
	     original_opts[num_orig_opts].name; 
	     num_orig_opts++);

	for (num_new_opts = 0;
	     new_options[num_new_opts].name;
	     num_new_opts++);

	opt = malloc(sizeof(struct option) * (num_new_opts + num_orig_opts + 1));
	memcpy(opt, original_opts, num_orig_opts * sizeof(struct option));
	memcpy(opt + num_orig_opts, new_options, 
	       (num_new_opts) * sizeof(struct option));

	return opt;
}

