/* Header for use in defining a given protocol. */
#ifndef _IP_NAT_PROTO_H
#define _IP_NAT_PROTO_H

#include "ip_nat_rule.h"
#include "ip_nat_binding.h"

struct ip_nat_proto
{
	/* Next pointer. */
	struct ip_nat_proto *next;

	/* Protocol number. */
	unsigned int proto;

	/* Protocol name */
	const char *name;

	/* Try to fill in the third arg; return true if possible. */
	int (*pkt_to_tuple)(const void *datah, 
			    size_t datalen, 
			    struct ip_nat_tuple *tuple);
	/* Do a packet translation according to the ip_nat_proto_manip
	 * and manip type. */
	void (*manip_pkt)(struct iphdr *hdr, 
			  size_t len,
			  const struct ip_nat_tuple_manip *manip,
			  enum ip_nat_manip_type manip_type);

	/* Is the manipable part of the tuple between min and max incl? */
	int (*in_range)(const struct ip_nat_tuple_proto *tuplepr,
			enum ip_nat_manip_type manip_type,
			const union ip_nat_proto_manip *min,
			const union ip_nat_proto_manip *max);

	/* Alter the per-proto part of the tuple (depending on
	   HOOK2MANIP), to give a unique tuple in the given range if
	   possible; return false if not.  Per-protocol part of tuple
	   is initialized to the incoming packet. */
	int (*unique_tuple)(struct ip_nat_tuple_proto *tuplepr,
			    const struct ip_nat_range *range,
			    unsigned int hook);

	/* Invert the per-proto part of the tuple: turn xmit into reply. */
	void (*invert_tuple)(struct ip_nat_tuple *tuple);

	/* Print out the per-protocol part of the tuple. */
	unsigned int (*print_tuple)(char *buffer, const struct ip_nat_tuple *);

	/* Print out the per-protocol part of the manip. */
	unsigned int (*print_manip)(char *buffer, 
				    const union ip_nat_proto_manip *);

	/* Standard rule functions for this protocol. */
	struct ip_nat_rule_type ruletype;
};

/* Protocol registration. */
extern int ip_nat_register_proto(struct ip_nat_proto *proto);
extern int ip_nat_unregister_proto(struct ip_nat_proto *proto);

#endif /*_IP_NAT_PROTO_H*/
