/* Kernel module to match MAC address parameters. */
#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/if_ether.h>

#include "ipt_mac.h"
#include "ip_tables.h"

static int 
match(struct sk_buff *skb,
      const char *indev,
      const char *outdev,
      const union ipt_matchinfo *matchinfo,
      int offset,
      const void *hdr,
      u_int16_t datalen)
{
    const struct ipt_mac_info *info = (void *)matchinfo->other;

    /* Is mac pointer valid? */
    return (skb->mac.raw >= skb->head 
	    && skb->mac.raw < skb->head + skb->len - ETH_HLEN
	    /* If so, compare... */
	    && memcmp(skb->mac.ethernet->h_source, info->srcaddr, ETH_ALEN)
	    == 0);
}

static struct ipt_match mac_match
= { NULL, "mac", &match, NULL, &__this_module };

int init_module(void)
{
	return ipt_register_match(&mac_match);
}

void cleanup_module(void)
{
	ipt_unregister_match(&mac_match);
}
