#ifndef _LIBPKTMATCH_H
#define _LIBPKTMATCH_H

/* A framework for doing packet data mangling on a TCP connection. */

struct iphdr;

/* A pattern consists of a linked list of these. */
/* Note that consecutive ones of these must not overlap; ie [0-1] and
[2-4] is ok, but [0-2] and [2-4] is illegal. */
struct pkm_pattern
{
	struct pkm_pattern *next;
	unsigned int num_possibilities;
	unsigned char *possibilities;
	unsigned int minimum_repeats, maximum_repeats;
};

/* The main loop: returns 0 on fail, 1 on successful completion. */
/* pattern: data pattern to look for.
   get_packet: function to get packet, returns -1 on error or pkt len,
	sets dir to 1 if it's in the substitution dir, 0 if not.
   get_packet_info: last arg to get_packet.
   replacement: function to get replacement for given pattern; returns
	pointer to buffer and sets replacement_length.
   replacement_info: last arg to replacement.
   reinject_packet: function to let packet continue, returns 1 on success.
   reinject_info: last arg to reinject_packet.
*/
extern int pkm_loop(const struct pkm_pattern *pattern,
		    int (*get_packet)(struct iphdr *pkt, void *info, int *dir),
		    void *get_packet_info,
		    unsigned char *(*replacement)(unsigned char *pattern_start,
						  size_t pattern_length,
						  size_t *replacement_length,
						  void *info),
		    void *replacement_info,
		    int (*reinject_packet)(struct iphdr *pkt, size_t len,
					   void *info),
		    void *reinject_info,
		    int (*discard_packet)(struct iphdr *pkt, size_t len,
					  void *info),
		    void *discard_info);
#endif /*_LIBPKTMATCH_H*/
