/* Simple program to test replacement. */


#include <sys/types.h>
#include <net/if.h>

#include <linux/netfilter_dev.h>
#include <linux/netfilter_ipv4.h>

#include <fcntl.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include "libpktmatch.h"

struct iphdr;

#define NETFILTER_DEVICE_NAME 	"/dev/netfilter_ipv4"

static int 
get_packet(struct iphdr *pkt, void *info, int *dir)
{
	struct nfdev_head head;
	int fd = (int)info;

	if (read(fd, &head, sizeof(head)) == sizeof(head)) {
		printf("in=%s, out=%s, mk=%u, len=%u\n",
		       head.iniface, head.outiface, head.mark, head.pktlen);
		if (head.mark == 1)
			*dir = 1;
		else
			*dir = 0;

		if (read(fd, pkt, head.pktlen) != head.pktlen)
			return 0;
		return head.pktlen;
	}
	else return 0;

}

static unsigned char *
replacement(unsigned char *pattern_start,
	    size_t pattern_length,
	    size_t *replacement_length,
	    void *info)
{
	size_t i;

	/* info contains a nul-terminated string with the replacement
           in it. */
	printf("Replacing `");
	for (i = 0; i < pattern_length; i++) 
		fputc(pattern_start[i], stdout);
	printf("' with `%s'\n", (char *)info);
	*replacement_length = strlen(info);
	return info;
}

static int reinject_packet(struct iphdr *pkt, size_t len, void *info)
{
	int fd = (int)info;
	struct nfdev_verdict verdict = { NF_ACCEPT, len, 1, 0 };

	printf("reinjecting packet.\n");
	if (ioctl(fd, NFDIOSVERDICT, &verdict) != 0) {
		fprintf(stderr, "reinject: %s\n", strerror(errno));
		return 0;
	}
	return 1;
}

static int discard_packet(struct iphdr *pkt, size_t len, void *info)
{
	int fd = (int)info;
	struct nfdev_verdict verdict = { NF_DROP, len, 1, 0 };

	printf("discarding packet.\n");
	if (ioctl(fd, NFDIOSVERDICT, &verdict) != 0) {
		fprintf(stderr, "discard: %s\n", strerror(errno));
		return 0;
	}
	return 1;
}

static struct pkm_pattern *
make_pattern(const char *arg)
{
	static struct pkm_pattern *ret = NULL, *end = NULL;

	/* Simple expression. [] => set. */
	while (*arg) {
		struct pkm_pattern *pat;

		if (*arg == '[' && strchr(arg, ']')) {
			*strchr(arg, ']') = '\0';
			if (end 
			    && end->num_possibilities == strlen(arg) - 1
			    && memcmp(arg, end->possibilities, strlen(arg)-1)
			    == 0) {
				end->maximum_repeats++;
				arg += strlen(arg) + 1;
				continue;
			}

			pat = malloc(sizeof(struct pkm_pattern) + strlen(arg));
			pat->num_possibilities = strlen(arg) - 1;
			memcpy(pat+1, arg+1, strlen(arg));
		} else {
			pat = malloc(sizeof(struct pkm_pattern) + 1);
			pat->num_possibilities = 1;
			*(char *)(pat + 1) = *arg;
		}
		pat->next = NULL;
		pat->possibilities = (unsigned char *)(pat + 1);
		pat->minimum_repeats = pat->maximum_repeats = 1;
		if (end)
			end->next = pat;
		else ret = end = pat;
		arg++;
	}

	return ret;
}

int main(int argc, const char *argv[])
{
	int fd;

	if (argc != 3) {
		fprintf(stderr, "Usage: %s [pattern] [replacement]\n", 
			argv[0]);
		return 1;
	}

	fd = open(NETFILTER_DEVICE_NAME, O_RDWR);
	if (fd < 0) {
		fprintf(stderr, "opening %s: %s\n", 
			NETFILTER_DEVICE_NAME, strerror(errno));
		return 1;
	}

	/* We catch everything, and use the following magic:
	 * mark = 1 means it's the direction to replace. */
	pkm_loop(make_pattern(argv[1]),
		 get_packet, (void *)fd,
		 replacement, (void *)argv[2],
		 reinject_packet, (void *)fd,
		 discard_packet, (void *)fd);

	return 0;
}

	

