/* Protocol handling.  Includes the standard "unknown" protocol, and
   the required "icmp" protocol directly. */

/* Hash table of registered protocols */
static struct ip_nat_proto *ip_nat_prototable[MAX_INET_PROTOS];

#include "protocols/ip_nat_unknown.c"
#include "protocols/ip_nat_icmp.c"

struct ip_nat_proto *
find_proto(unsigned int protocol)
{
	struct ip_nat_proto *ret;

	for (ret = ip_nat_prototable[protocol & (MAX_INET_PROTOS-1)];
	     ret && ret->proto != protocol;
	     ret = ret->next);

	return ret ?: &unknown_proto;
}

int ip_nat_register_proto(struct ip_nat_proto *proto)
{
	int ret;

	/* If any optional are null, substitute in the "unknown"
           generic ones. */
	if (!proto->in_range)
		proto->in_range = unknown_proto.in_range;
	if (!proto->invert_tuple)
		proto->invert_tuple = unknown_proto.invert_tuple;
	if (!proto->print_tuple)
		proto->print_tuple = unknown_proto.print_tuple;
	if (!proto->print_manip)
		proto->print_manip = unknown_proto.print_manip;

	ret = ip_nat_register_mapping_type(&proto->mappingtype);

	if (ret == 0) {
		/* We don't want two increments for a protocol; don't
                   inc mod usage here.. */
		modreg_prepend(&ip_nat_prototable[proto->proto
						 & (MAX_INET_PROTOS-1)],
			       proto);
	}
	return ret;
}
	
int ip_nat_unregister_proto(struct ip_nat_proto *proto)
{
	DEBUGP("Deleting proto `%s'\n", proto->name);
	modreg_delete(&ip_nat_prototable[proto->proto & (MAX_INET_PROTOS-1)],
		      proto);
	DEBUGP("Deleted proto `%s'\n", proto->name);
	return ip_nat_unregister_mapping_type(&proto->mappingtype);
}

void init_protos(void)
{
	ip_nat_register_proto(&proto_icmp);
	MOD_DEC_USE_COUNT;
}

void cleanup_protos(void)
{
	MOD_INC_USE_COUNT;
	ip_nat_unregister_proto(&proto_icmp);
}





