/* Kernel module to match UDP parameters. */
#include <linux/module.h>
#include <linux/skbuff.h>
#include <linux/udp.h>

#include "ipt_udp.h"
#include "ip_tables.h"

/* Returns 1 if the port is matched by the range, 0 otherwise */
static inline int
port_match(u_int16_t min, u_int16_t max, u_int16_t port, int invert)
{
	return (port >= min && port <= max) ^ invert;
}

static int
match(struct sk_buff *skb,
      const char *indev,
      const char *outdev,
      const union ipt_matchinfo *matchinfo,
      int offset,
      const void *hdr,
      u_int16_t datalen)
{
	const struct udphdr *udp = hdr;

	/* Must be long enough to examine, and not a fragment. */
	return datalen >= sizeof(struct udphdr)
		&& !offset
		&& port_match(matchinfo->udp.spts[0], matchinfo->udp.spts[1],
			      ntohs(udp->source),
			      !!(matchinfo->udp.invflags & IPT_UDP_INV_SRCPT))
		&& port_match(matchinfo->udp.dpts[0], matchinfo->udp.dpts[1],
			      ntohs(udp->dest),
			      !!(matchinfo->udp.invflags & IPT_UDP_INV_DSTPT));
}

/* Called when user tries to insert an entry of this type. */
static int
checkentry(const struct ipt_ip *ip,
	   const union ipt_matchinfo *matchinfo)
{
	/* Must specify proto == UDP, and no unknown invflags */
	return ip->proto == IPPROTO_UDP
		&& !(matchinfo->udp.invflags & ~IPT_UDP_INV_MASK);
}

static struct ipt_match udp_match
= { NULL, "udp", &match, &checkentry, &__this_module };

int init_module(void)
{
	return ipt_register_match(&udp_match);
}

void cleanup_module(void)
{
	ipt_unregister_match(&udp_match);
}
