/* Shared library add-on to iptables to add marking support. */
#include <stdio.h>
#include <getopt.h>
#include <stdlib.h>
#include <limits.h>
#include "iptables.h"
#include "ip_tables.h"
#include "ipt_MARK.h"

/* Function which prints out usage message. */
static void
help(void)
{
	printf(
"MARK options:\n"
" --mark mark		Set mark to the given value.\n");
}

static struct option opts[] = {
	{ "mark", 1, 0, '!' },
	{ 0 }
};

static void
init(union ipt_targinfo *targinfo)
{
	struct ipt_mark_info *info = (struct ipt_mark_info *)targinfo->other;

	memset(info, 0, sizeof(info));
}

static void
parse_mark(const char *markstring, u_int32_t *mark)
{
	char *end;
	unsigned long l;

	l = strtoul(markstring, &end, 0);

	if (end[0] != '\0' || end == markstring || l > UINT_MAX)
		exit_error(PARAMETER_PROBLEM, "Bad value `%s' for --mark.\n",
			   markstring);
	*mark = l;
}

/* Function which parses command options; returns true if it
   ate an option */
static int
parse(int c, char **argv, int invert, unsigned int *flags,
      struct ipt_entry *entry)
{
	struct ipt_mark_info *info = 
		(struct ipt_mark_info *)entry->targinfo.other;

	switch (c) {
	case '!':
		if (check_inverse(optarg, &invert))
			exit_error(PARAMETER_PROBLEM,
				   "! not allowed with --mark");

		parse_mark(optarg, &info->mark);
		break;

	default:
		return 0;
	}

	return 1;
}

/* Final check; nothing. */
static void final_check(unsigned int flags)
{
}

/* Prints out the union ipt_targinfo. */
static void print(const union ipt_targinfo *info, int numeric)
{
	printf("MARK %u ", ((struct ipt_mark_info *)info->other)->mark);
}

struct iptables_target mark
= { NULL,
    "MARK",
    &help,
    &init,
    &parse,
    &final_check,
    &print,
    opts
};

void _init(void)
{
	register_target(&mark);
}
