/* Generic functions to manage a system where modules want to reserve
   private space in your structure. */

#ifndef _STRUCT_RESERVE_H
#define _STRUCT_RESERVE_H

#define _STRUCT_CAT(x,y) x ## y

/* Hand these the type name, and the space you require.  Returns
   offset from start of structure. */
#define struct_reserve(type, space)				\
	(__struct_reserve(&_STRUCT_CAT(reserve_,type), space)	\
	 + sizeof(struct type))

#define struct_unreserve(type, space)				\
	(__struct_unreserve(&_STRUCT_CAT(reserve_,type), space)	\
	 + sizeof(struct type))

#define EXTERN_DECLARE_STRUCT_RESERVABLE(type)		\
	extern size_t _STRUCT_CAT(reserve_,type)

#define DECLARE_STRUCT_RESERVABLE(type)		\
	size_t _STRUCT_CAT(reserve_,type) = 0

#define STRUCT_RESERVE_ALLOCATE(type, pri)			\
	({ struct type *x = kmalloc(sizeof(struct type)		\
			    + _STRUCT_CAT(reserve_,type), pri);	\
	   if (x) memset(x+1, 0, _STRUCT_CAT(reserve_,type));	\
	   x; })

extern size_t 
__struct_reserve(size_t *extra, size_t size);
extern void
__struct_unreserve(size_t *extra, size_t offset, size_t size);
#endif /*_STRUCT_RESERVE_H*/
