#define __NO_VERSION__
#include <linux/types.h>
#include <linux/timer.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/netfilter.h>
#include "../state/ip_state.h"
#include "ip_nat.h"
#include "ip_nat_rule.h"
#include "ip_nat_protocol.h"

static struct ip_nat_protocol *protos = NULL;
extern void ip_nat_icmp_protocol_init();
extern void ip_nat_udp_protocol_init();
extern void ip_nat_tcp_protocol_init();

extern struct ip_nat_protocol unknown_protocol;

struct ip_nat_protocol *
find_proto(u_int16_t protonum)
{
	struct ip_nat_protocol *i;

	for (i = protos; i; i = i->next) 
		if (i->protonum == protonum)
			return i;

	return &unknown_protocol;
}

/* Protocol registration. */
int ip_nat_protocol_register(struct ip_nat_protocol *proto)
{
	if (!modreg_named_insert(&protos, proto))
		return -EEXIST;

	MOD_INC_USE_COUNT;
	return 0;
}

void ip_nat_protocol_unregister(struct ip_nat_protocol *proto)
{
	modreg_delete(&protos, proto);
	MOD_DEC_USE_COUNT;
}

static int
init_or_cleanup(int init)
{
	int ret = 0;

	if (!init) goto cleanup;

	/* These are prepended: TCP most popular, so put it first. */
	ret = ip_nat_protocol_icmp_init();
	if (ret < 0)
		goto cleanup_nothing;
	ret = ip_nat_protocol_udp_init();
	if (ret < 0)
		goto cleanup_icmp;
	ret = ip_nat_protocol_tcp_init();
	if (ret < 0)
		goto cleanup_udp;
	MOD_DEC_USE_COUNT;
	MOD_DEC_USE_COUNT;
	MOD_DEC_USE_COUNT;
	return ret;

 cleanup:
	MOD_INC_USE_COUNT;
	MOD_INC_USE_COUNT;
	MOD_INC_USE_COUNT;
	ip_nat_protocol_tcp_cleanup();
 cleanup_udp:
	ip_nat_protocol_udp_cleanup();
 cleanup_icmp:
	ip_nat_protocol_icmp_cleanup();
 cleanup_nothing:
	return ret;
}
__initfunc(int init_protocols(void))
{
	return init_or_cleanup(1);
}

void cleanup_protocols(void)
{
	init_or_cleanup(0);
}
