#define __NO_VERSION__
#include <linux/module.h>
#include <linux/netfilter.h>
#include <asm/spinlock.h>
#include "ip_state_protocol.h"

/* If we're the first tuple, it's the original dir. */
#define DIRECTION(h) (&(h)->state->tuplehash[1] == (h))

/* Built-in protocols. */
extern struct ip_state_protocol ip_state_generic_protocol;
extern void ip_state_icmp_protocol_init();
extern void ip_state_udp_protocol_init();
extern void ip_state_tcp_protocol_init();

static struct ip_state_protocol *protocol_list = NULL;

static int cmpfn(const void *arg, const void *curr)
{
	/* We cast to unsigned long because gcc bitches if we cast to
           u_int8_t */
	return (unsigned long)arg == ((struct ip_state_protocol *)curr)->proto;
}

struct ip_state_protocol *find_proto(u_int8_t protocol)
{
	struct ip_state_protocol *p
		= modreg_find(&protocol_list, (void *)(unsigned long)protocol,
			      &cmpfn);

	if (!p)
		p = &ip_state_generic_protocol;
	return p;
}

/* FIXME: Allow NULL functions and sub in pointers to generic for
   them. --RR */
int ip_state_register_protocol(struct ip_state_protocol *proto)
{
	MOD_INC_USE_COUNT;
	if (modreg_find(&protocol_list, (void *)(unsigned long)proto->proto,
			&cmpfn))
		return 0;
	modreg_prepend(&protocol_list, proto);
	return 1;
}

void ip_state_unregister_protocol(struct ip_state_protocol *proto)
{
	unsigned int i;
	unsigned long flags;
	
	modreg_delete(&protocol_list, proto);

	spin_lock_irqsave(&ip_state_lock, flags);

	/* Delete those using us.  They'll fall back to generic. */
	/* Traverse hash. */
	for (i = 0; i < IP_STATE_HTABLE_SIZE; i++) {
		struct ip_state_tuple_hash *h;

		for (h = ip_state_hash[i]; h; ) {
			struct ip_state_tuple_hash *next = h->next;

			if (h->tuple.dst.protonum == proto->proto)
				ip_state_delete_maybe(h->state, DIRECTION(h));

			h = next;
		}
	}
	spin_unlock_irqrestore(&ip_state_lock, flags);
	MOD_DEC_USE_COUNT;
}

/* Sews in the default protocols. */
__initfunc(void ip_state_protocols_init(void))
{
	/* These are prepended: TCP most popular, so put it first. */
	ip_state_icmp_protocol_init();
	ip_state_udp_protocol_init();
	ip_state_tcp_protocol_init();
	MOD_DEC_USE_COUNT;
	MOD_DEC_USE_COUNT;
	MOD_DEC_USE_COUNT;
}
