#ifndef _LIBPERPROTO_H
#define _LIBPERPROTO_H
/* Helper library for per-protocol helpers. */

/* Register to handle a port (network order); returns -1 or
 * netfilter_dev file handle.  If handle is -1, opens
 * /dev/netfilter_ipv4, otherwise tries to attach that protocol to the
 * handle given.  */
int 
perproto_register(u_int8_t proto, u_int16_t port, int direction, int handle);

typedef struct pp_pattern *perproto_pattern_handle;

/* Register interest for a given data pattern. */
perproto_pattern_handle 
perproto_pattern_register(const void *pattern, size_t size)

/* "" ensure it's a string literal, otherwise parse error. */
#define perproto_pattern_register_string(s)		\
   perproto_pattern_register(s "", sizeof(s) - 1)

/* Returns -1 or offset in packet after match. */
int
perproto_pattern_check(perproto_pattern_handle handle,
		       const unsigned char *data, u_int32_t len);

/* Unregister interest in a given pattern */
void
perproto_pattern_unregister(perproto_pattern_handle handle);

/* Given this protocol, srcip, dstip, srcpt and dstpt (network order),
   and direction, return the replacement ip/port.  Returns bool. */
int 
perproto_getorig(u_int8_t proto, u_int32_t srcip, u_int32_t dstip,
		 u_int16_t srcpt, u_int16_t dstpt, int direction,
		 u_int32_t *ip, u_int16_t *pt);

int
perproto_natonce(u_int8_t proto, u_int32_t srcip, u_int32_t dstip,
		 u_int16_t srcpt, u_int16_t dstpt, int direction,
		 u_int32_t newip, u_int16_t newpt);
#endif /* _LIBPERPROTO_H */
