/* Masquerade.  Simple setup which alters range to a local IP address
   (depending on route) before calling the next setup. */
#include <linux/types.h>
#include <linux/ip.h>
#include <linux/timer.h>
#include <linux/modreglist.h>
#include <linux/module.h>
#include <linux/netfilter.h>
#include <net/protocol.h>
#include <net/checksum.h>
#include "ip_nat_bind.h"

static unsigned int 
masquerade_print(char *buffer, const struct ip_nat_perconn *perconn)
{
	return 0;
}

static int masquerade_check(const struct ip_nat_binding *bind,
			    unsigned int typenumber)
{
	/* We can't be the final one. */
	if (!bind->elem.type[typenumber+1]) {
		DEBUGP("ipnat_bind_masquerade_check: can't be final.\n");
		return 0;
	}

	if (bind->elem.dir != IP_NAT_OUTGOING) {
		DEBUGP("ipnat_bind_masquerade_check: must be OUTGOING.\n");
		return 0;
	}

	return 1;
}

/* FIXME: If we have a linked list of ranges, this only does the first
   one. --RR */
static unsigned int masquerade_bind(const struct iphdr *iph,
				    const struct ip_nat_pktinfo *pkt,
				    const struct ip_nat_binding_element *cause,
				    struct ip_nat_multi_range *range,
				    unsigned int typenumber,
				    struct ip_nat_perconn **perconn)

{
	unsigned int ret;
	struct ip_nat_multi_range oldrange = *range;

	/* Don't do anything for locally generated packets. */
	if (pkt->device) {
		struct rtable *rt;

		/* Pass 0 instead of saddr, since it's going to be
                   changed anyway. */
		/* FIXME: Maybe pass dev->ifindex (or dev->iflink?)
		   instead of 0? --RR */
		if (ip_route_output(&rt, iph->daddr, 0, IPTOS_TOS(iph->tos), 0)
		    != 0) {
			DEBUGP("ipnat_bind_masquerade: Can't reroute.\n");
			return NF_DROP;
		}

		range->range.min_ip = range->range.max_ip
			= inet_select_addr(rt->u.dst.dev, rt->rt_gateway, 
					   RT_SCOPE_UNIVERSE);
		range->next = NULL;

		ip_rt_put(rt);
	}

	ret = cause->type[typenumber+1]->bind(iph, pkt, cause,
					      range, typenumber+1, perconn);
	/* FIXME: Copy range_full back? --RR */
	*range = oldrange;
	return ret;
}

/* Don't care if they destroy a perconn. */
static void masquerade_destroy(const struct ip_nat_perconn *perconn)
{
}

static struct ip_nat_bind_type bind_masquerade
= { NULL, "masquerade", 0, &__this_module, 0, 0, 
    masquerade_print, 
    masquerade_check, 
    masquerade_bind,
    masquerade_destroy };

int init_module(void)
{
	return ip_nat_register_bind_type(&bind_masquerade);
}

void cleanup_module(void)
{
	ip_nat_unregister_bind_type(&bind_masquerade);
}
