/* Redirect.  Simple setup which alters range to a local IP address
   before calling the next setup. */
#include <linux/types.h>
#include <linux/ip.h>
#include <linux/timer.h>
#include <linux/modreglist.h>
#include <linux/module.h>
#include <net/protocol.h>
#include <net/checksum.h>

#include "ip_nat.h"
#include "ip_nat_bind.h"

static unsigned int 
redirect_print(char *buffer,
	       const struct ip_nat_perconn *perconn)
{
	return 0;
}

static int redirect_check(const struct ip_nat_binding *bind,
			  unsigned int typenumber)
{
	/* We can't be the final one. */
	if (!bind->elem.type[typenumber+1]) {
		DEBUGP("ipnat_bind_redirect_check: can't be final.\n");
		return 0;
	}

	if (bind->elem.dir != IP_NAT_INCOMING) {
		DEBUGP("ipnat_bind_redirect_check: must be INCOMING.\n");
		return 0;
	}

	return 1;
}

/* FIXME: If we have a linked list of ranges, this only does the first
   one. --RR */
static unsigned int redirect_bind(const struct iphdr *iph,
			     const struct ip_nat_pktinfo *pkt,
			     const struct ip_nat_binding_element *cause,
			     struct ip_nat_multi_range *range,
			     unsigned int typenumber,
			     struct ip_nat_perconn **perconn)
{
	unsigned int ret;
	struct ip_nat_multi_range oldrange = *range;

	/* Local packets: make them go to loopback */
	if (!pkt->device)
		range->range.max_ip = range->range.min_ip = htonl(0x7F000001);
	else
		/* Grab first address on interface. */
		range->range.max_ip = range->range.min_ip 
			= (((struct in_device *)pkt->device->ip_ptr)
			   ->ifa_list->ifa_local);

	range->next = NULL;

	ret = cause->type[typenumber+1]->bind(iph, pkt, cause,
					      range, typenumber+1, perconn);
	*range = oldrange;
	return ret;
}

/* Don't care if they destroy a perconn. */
static void redirect_destroy(const struct ip_nat_perconn *perconn)
{
}

static struct ip_nat_bind_type bind_redirect
= { NULL, "redirect", 0, &__this_module, 0, 0, 
    redirect_print, 
    redirect_check, 
    redirect_bind,
    redirect_destroy };

int init_module(void)
{
	return ip_nat_register_bind_type(&bind_redirect);
}

void cleanup_module(void)
{
	ip_nat_unregister_bind_type(&bind_redirect);
}
